// test-SigApplication
//
// Copyright © 1998 Arcane Dragon Software
//                  Chris Herborth (chrish@kagi.com)
//
// Test the ChSigApplication.

#include <interface/Window.h>
#include <interface/View.h>
#include <stdio.h>
#include <signal.h>

#include "Ch/ChSigApplication.h"

static char *_rcs_id = "$Id: test-SigApplication.cpp,v 1.2 1998/04/14 00:34:05 chrish Exp $";

// ----------------------------------------------------------------------
// Our view object.
class SigView : public BView {
public:
	SigView( BRect frame, const char *name, uint32 resizing, uint32 flags )
		: BView( frame, name, resizing, flags ) { ; }
	virtual ~SigView( void ) { ; }
	
	virtual void Draw( BRect update );
};

void SigView::Draw( BRect update )
{
#pragma unused (update)
	MovePenTo( 10.0, 25.0 );
	DrawString( "Do a “kill -INT test-SigApplication” in a Terminal!" );
}

// ----------------------------------------------------------------------
// Our application object.
class SigApplication : public ChSigApplication {
public:
	SigApplication( sigset_t *set = NULL ) 
		: ChSigApplication( "application/x-vnd.ads-sigapptest", set ) {
		_wind = new BWindow( BRect( 100, 100, 600, 200 ), "SigApplication test",
						B_TITLED_WINDOW, 
						B_NOT_RESIZABLE | B_NOT_ZOOMABLE | B_NOT_MINIMIZABLE );
		if( _wind ) {
			BView *view = new SigView( _wind->Bounds(), "Sig view",
								B_FOLLOW_ALL_SIDES, B_WILL_DRAW );
			if( view ) {
				_wind->AddChild( view );
			}

			_wind->Show();
		}
	}

	virtual ~SigApplication( void ) { ; }

	// ChSigApplication methods we'd like to override.	
	virtual void SignalReceived( int signo );
	virtual void QuitDemanded( void );

private:
	BWindow *_wind;
};

void SigApplication::SignalReceived( int signo )
{
	switch( signo ) {
	case SIGINT:
		printf( "Got a SIGINT!\n" );
		break;

	case SIGHUP:
		printf( "Got a SIGHUP!\n" );
		break;

	case SIGWINCH:
		printf( "This Terminal's window size just changed!\n" );
		
		// We'll ignore this one.
		return;
		break;

	defult:
		printf( "Got some kind of signal.\n" );
		break;
	}

	// All the other signals will go to the default handler.
	ChSigApplication::SignalReceived( signo );
}

void SigApplication::QuitDemanded( void )
{
	printf( "Woah, this application is outta here!\n" );

	ChSigApplication::QuitDemanded();
}

// ----------------------------------------------------------------------
// Mainline
int main( void )
{
	printf( "ChSigApplication test\n" );
	printf( "%s\n", _rcs_id );
	printf( "\n" );

	// Now create our app and let it run.
	sigset_t handle_set;
	(void)sigfillset( &handle_set );

	SigApplication app( &handle_set );
	if( app.IsValid() ) {
		app.Run();
	} else {
		printf( "This sucks... the application isn't valid.\n" );
	}

	return 0;
}
