// test-Snoozer
//
// Copyright © 1997-1998 Arcane Dragon Software
//                       Chris Herborth (chrish@kagi.com)
//
// Test the ChSnoozer object by throwing around a few threads.
//
// As an exercise for the reader, you could modify this to spawn n threads
// based on a command-line option...

#include <kernel/OS.h>
#include <stdio.h>

#include "Ch/ChBenaphore.h"
#include "Ch/ChSnoozer.h"

static char *_rcs_id = "$Id: test-Snoozer.cpp,v 1.2 1998/04/14 00:34:05 chrish Exp $";

// Prototypes.
long thread1( void *data );

// A global to smooth access to stdout.
ChBenaphore *stdout_guard;
ChBenaphore *starter;

// Print to stdout, one at a time.
static void safe_puts( const char *str )
{
	stdout_guard->Lock();
	puts( str );
	stdout_guard->Unlock();
}

// Thread 1
long thread1( void *data )
{
	ChSnoozer *sleepy = (ChSnoozer *)data;

	safe_puts( "--- 1 - ready to sleep              ---" );

	bigtime_t before = system_time();
	do {
		safe_puts( "--- 1 - sleeping for 60 seconds     ---" );
	} while( sleepy->SleepFor( 1000 * 1000 * 60 ) );
	bigtime_t after = system_time();

	char buff[512];
	sprintf( buff, "--- 1 - slept for about %02d seconds ---",
			 (long)( ( after - before ) / 1000 / 1000 ) );
	safe_puts( buff );

	return 0;
}

// main
int main( void )
{
	printf( "ChSnoozer test\n" );
	printf( "%s\n", _rcs_id );
	printf( "\n" );

	stdout_guard = new ChBenaphore( "stdout guard" );
	if( !stdout_guard->IsValid() ) {
		fprintf( stderr, "Can't create ChBenaphore to guard stdout...\n" );
		return -1;
	}

	ChSnoozer the_snoozer( "main's snoozer" );

	// Now we create some threads and let them fight over the ChBenaphore.
	thread_id t1 = spawn_thread( thread1,
								 "Thread 1",
								 B_NORMAL_PRIORITY,
								 (void *)&the_snoozer );
	if( t1 < B_NO_ERROR ) {
		fprintf( stderr, "main(): Argh, couldn't spawn our test threads.\n" );
		return -1;
	}

	status_t retval = resume_thread( t1 );
	if( retval != B_NO_ERROR ) {
		fprintf( stderr, "main(): Couldn't start thread 1.\n" );
		return -1;
	}

	// Let 'em sleep for 10 seconds.
	safe_puts( "main(): waiting for 10 seconds..." );
	snooze( 10 * 1000 * 1000 );

	// Kill the threads by waking them up.
	safe_puts( "main(): waking up the snoozer..." );
	retval = the_snoozer.WakeUp();
	if( retval != B_NO_ERROR ) {
		fprintf( stderr, "main(): Error (%d) waking thread 1!\n", retval );
	}

	// Clean up.
	stdout_guard->Lock();
	delete stdout_guard;

	// Bye bye now.
	return 0;
}
